
#include <stdio.h>
#include <Windows.h>

#include "interface.h"
#include "filesystem.h"
#include "engine/iserverplugin.h"
#include "game/server/iplayerinfo.h"
#include "eiface.h"
//#include "igameevents.h"
#include "convar.h"
//#include "networkstringtabledefs.h"
#include "Color.h"
//#include "vstdlib/random.h"
#include "engine/IEngineTrace.h"
#include "tier2/tier2.h"

#include "game.h"

#include "engine/ivmodelinfo.h"
#include "studio.h"
#include "materialsystem/imaterial.h"
#include "vphysics_interface.h"
#include "Common/ILuaInterface.h"

// memdbgon must be the last include file in a .cpp file!!!
#include "tier0/memdbgon.h"

// Engine interfaces.
IVEngineServer * engine = NULL;
IVModelInfo * modelinfo = NULL;

ILuaInterface * gLua = NULL;

int getmodelproperties( lua_State * L )
{
	if( gLua->Top() == 0 || gLua->GetType( 1 ) != GLua::TYPE_STRING )
	{
		gLua->TypeError( "String", 1 );
		return 0;
	}

	const char * pModelName = gLua->GetString( 1 );

	int modelindex = modelinfo->GetModelIndex( pModelName );

	if( modelindex < 0 )
	{
		gLua->Push( "bad model index" );
		return 1;
	}

	const model_t * pMdl = modelinfo->GetModel( modelindex );

	if( pMdl == NULL )
	{
		gLua->Push( "bad model_t pointer" );
		return 1;
	}

	studiohdr_t * pStdHdr = modelinfo->GetStudiomodel( pMdl );

	if( pStdHdr == NULL )
	{
		gLua->Push( "bad studiohdr_t pointer" );
		return 1;
	}

	gLua->Push( pStdHdr->pszSurfaceProp() );
	gLua->Push( pStdHdr->mass );

	return 2;
}

extern "C" __declspec( dllexport ) int gmod_open( ILuaInterface * i )
{
	CreateInterfaceFn engineFactory = Sys_GetFactory( "engine.dll" );

	if( !engineFactory )
	{
		Warning( " * surfaceprops: Could not load engine factory.\n" );
		return 0;
	}

	ConnectTier1Libraries( &engineFactory, 1 );
	ConnectTier2Libraries( &engineFactory, 1 );

	engine = (IVEngineServer*)engineFactory(INTERFACEVERSION_VENGINESERVER, NULL);
	modelinfo = (IVModelInfo *)engineFactory(VMODELINFO_SERVER_INTERFACE_VERSION, NULL);

	if( ! ( engine && modelinfo ) )
	{
		Warning( " * surfaceprops: Could not load some engine interface.\n" );

		Warning( "\tengine: %08x\n", (uint)engine );
		Warning( "\tmodelinfo: %08x\n", (uint)modelinfo );

		return 0;
	}

	MathLib_Init( 2.2f, 2.2f, 0.0f, 2.0f );
	ConVar_Register( 0 );

	gLua = i;

	gLua->SetGlobal( "getmodelproperties", getmodelproperties );

	//Msg( " * surfaceprops: It appears we've loaded okay.\n" );

	return 0;
}

extern "C" int __declspec( dllexport ) gmod_close( int i )
{
	return 0;
}